//+------------------------------------------------------------------+
//|                                             Sessions - Ahsan.mq4 |
//+------------------------------------------------------------------+

#property indicator_chart_window

extern int    NumberOfDays = 99;             
extern string AsiaBegin    = "01:00";       
extern string AsiaEnd      = "10:00";        
extern color  AsiaColor    = C'0,32,0';      
extern string EurBegin     = "08:00";        
extern string EurEnd       = "18:00";        
extern color  EurColor     = C'48,0,0';      
extern string USABegin     = "14:00";        
extern string USAEnd       = "23:00";        
extern color  USAColor     = C'0,0,56';      
extern bool   ShowPrice    = true;          
extern color  clFont       = White;           
extern int    SizeFont     = 8;              
extern int    OffSet       = 10;             

string        IndiName;

//+------------------------------------------------------------------+
void init() {
//+------------------------------------------------------------------+

  IndiName = "Sessions";
  IndicatorShortName(IndiName);

  DeleteObjects();
  for (int i=0; i<NumberOfDays; i++) {
    CreateObjects(IndiName+"-AS-"+i, AsiaColor);
    CreateObjects(IndiName+"-EU-"+i, EurColor);
    CreateObjects(IndiName+"-US-"+i, USAColor);
  }
  Comment("");
}

//+------------------------------------------------------------------+
void deinit() {
//+------------------------------------------------------------------+
  DeleteObjects();
  Comment("");
}

//+------------------------------------------------------------------+
void CreateObjects(string no, color cl) {
//+------------------------------------------------------------------+
  ObjectCreate(no, OBJ_RECTANGLE, 0, 0,0, 0,0);
  ObjectSet(no, OBJPROP_STYLE, STYLE_SOLID);
  ObjectSet(no, OBJPROP_COLOR, cl);
  ObjectSet(no, OBJPROP_BACK, True);
}

//+------------------------------------------------------------------+
void DeleteObjects() {
//+------------------------------------------------------------------+
  int k=0;
  while (k<ObjectsTotal())   {
    string objname = ObjectName(k);
    if (StringSubstr(objname,0,StringLen(IndiName)) == IndiName)  
      ObjectDelete(objname);
    else
      k++;
  }    
  return(0);
}

//+------------------------------------------------------------------+
void start() {
//+------------------------------------------------------------------+
  datetime dt=CurTime();
  
  for (int i=0; i<NumberOfDays; i++) {
//    if (ShowPrice && i==0) {
    if (ShowPrice) {
      DrawPrices(dt, IndiName+"-AS-prc-"+i, AsiaBegin, AsiaEnd);
      DrawPrices(dt, IndiName+"-EU-prc-"+i, EurBegin, EurEnd);
      DrawPrices(dt, IndiName+"-US-prc-"+i, USABegin, USAEnd);
    }
    DrawObjects(dt, IndiName+"-AS-"+i, AsiaBegin, AsiaEnd);
    DrawObjects(dt, IndiName+"-EU-"+i, EurBegin, EurEnd);
    DrawObjects(dt, IndiName+"-US-"+i, USABegin, USAEnd);
    dt=decDateTradeDay(dt);
    while (TimeDayOfWeek(dt)>5) dt=decDateTradeDay(dt);
  }
}

//+------------------------------------------------------------------+
void DrawObjects(datetime dt, string no, string tb, string te) {
//+------------------------------------------------------------------+
  datetime t1, t2;
  double   p1, p2;
  int      b1, b2;

  t1=StrToTime(TimeToStr(dt, TIME_DATE)+" "+tb);
  t2=StrToTime(TimeToStr(dt, TIME_DATE)+" "+te);
  b1=iBarShift(NULL, 0, t1);
  b2=iBarShift(NULL, 0, t2);
  p1=High[Highest(NULL, 0, MODE_HIGH, b1-b2, b2)];
  p2=Low [Lowest (NULL, 0, MODE_LOW , b1-b2, b2)];
  ObjectSet(no, OBJPROP_TIME1 , t1);
  ObjectSet(no, OBJPROP_PRICE1, p1);
  ObjectSet(no, OBJPROP_TIME2 , t2);
  ObjectSet(no, OBJPROP_PRICE2, p2);
}

//+------------------------------------------------------------------+
void DrawPrices(datetime dt, string no, string tb, string te) {
//+------------------------------------------------------------------+
  datetime t1, t2;
  double   p1, p2;
  int      b1, b2;

  t1=StrToTime(TimeToStr(dt, TIME_DATE)+" "+tb);
  t2=StrToTime(TimeToStr(dt, TIME_DATE)+" "+te);
  b1=iBarShift(NULL, 0, t1);
  b2=iBarShift(NULL, 0, t2);
  p1=High[Highest(NULL, 0, MODE_HIGH, b1-b2, b2)];
  p2=Low [Lowest (NULL, 0, MODE_LOW , b1-b2, b2)];

  double p3=p1-p2;
  int    dig=0;
  if (Digits==3 || Digits==5)  {
    p3/=10;
    dig=1;
  }  

  if (ObjectFind(no+"up")<0) ObjectCreate(no+"up", OBJ_TEXT, 0, 0,0);
  ObjectSet(no+"up", OBJPROP_TIME1   , t2);
  ObjectSet(no+"up", OBJPROP_PRICE1  , p1+OffSet*Point);
  ObjectSet(no+"up", OBJPROP_COLOR   , clFont);
  ObjectSet(no+"up", OBJPROP_FONTSIZE, SizeFont);
  ObjectSetText(no+"up", DoubleToStr(p1, Digits) + "  (" + DoubleToStr(p3/Point, dig) + ")");

  if (ObjectFind(no+"dn")<0) ObjectCreate(no+"dn", OBJ_TEXT, 0, 0,0);
  ObjectSet(no+"dn", OBJPROP_TIME1   , t2);
  ObjectSet(no+"dn", OBJPROP_PRICE1  , p2);
  ObjectSet(no+"dn", OBJPROP_COLOR   , clFont);
  ObjectSet(no+"dn", OBJPROP_FONTSIZE, SizeFont);
  ObjectSetText(no+"dn", DoubleToStr(p2, Digits));
}

//+------------------------------------------------------------------+
datetime decDateTradeDay (datetime dt) {
//+------------------------------------------------------------------+
  int ty=TimeYear(dt);
  int tm=TimeMonth(dt);
  int td=TimeDay(dt);
  int th=TimeHour(dt);
  int ti=TimeMinute(dt);

  td--;
  if (td==0) {
    tm--;
    if (tm==0) {
      ty--;
      tm=12;
    }
    if (tm==1 || tm==3 || tm==5 || tm==7 || tm==8 || tm==10 || tm==12) td=31;
    if (tm==2) if (MathMod(ty, 4)==0) td=29; else td=28;
    if (tm==4 || tm==6 || tm==9 || tm==11) td=30;
  }
  return(StrToTime(ty+"."+tm+"."+td+" "+th+":"+ti));
}

